# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""

order line item fulfillment, we defer full implementation of shippable fulfillment workflows

transition : create

   destination - new

transition : deliver-virtual

   source - new
   condition : not shippable
   state - delivered
   trigger : system

transition : ship

   source - new
   destination - processing

transition : cancel

   source - new
   state - cancelled

transition : cancel - processing

   source - processing
   state - cancelled
   condition - shipment not authorized
    
transition :  shipment-authorized

   source - processing
   state - shipppable

transition : process return

   source - shipped
   state - return in progress
   condition - returnable
   option - restocking fee   
   generate - rma number

   -- returns are there own shipments with workflow ?

transition : received return
  source : return in progress
  condition :
  option : ? 


transition : refund-delievered

   condition : virtual delivery | or refundable
   source : delivered
   state : refund-processing

transition : refund-processed

   state : refunded
   source : refund-processing
   

$Id: item.py 1141 2007-12-30 23:47:15Z kapilt $
"""

from zope.interface import implements
from zope import component

from hurry.workflow import interfaces as iworkflow
from hurry.workflow import workflow

from getpaid.core.interfaces import workflow_states, IShippableContent

import getpaid.core.workflow

from zope.i18nmessageid import MessageFactory
_ = MessageFactory('getpaid')

def VirtualDeliverable( wf, context ):
    return not component.queryAdapter( IShippableContent, context ) is None

def create_item_workflow( ):

    its = workflow_states.item

    transitions = []
    add = transitions.append
    
    add( workflow.Transition(
        transition_id = 'create',
        title='Create',
        source = None,
        destination = its.NEW
        ) )

    add( workflow.Transition(
        transition_id = 'deliver-virtual',
        title=_(u'Electronic Delivery'),
        condition = VirtualDeliverable,
        trigger = iworkflow.SYSTEM,
        source = its.NEW,
        destination = its.DELIVER_VIRTUAL
        ) )    

    add( workflow.Transition(
        transition_id = 'cancel',
        title=_(u'Cancel'),
        source = its.NEW,
        destination = its.CANCELLED
        ) )    


    add( workflow.Transition(
        transition_id = 'refund',
        title=_(u'Refund'),
        source = its.DELIVER_VIRTUAL,
        destination = its.REFUNDING
        ) )

    add( workflow.Transition(
        transition_id = 'refund-processed',
        title=_(u'Refund Processed'),
        source = its.REFUNDING,
        trigger = iworkflow.SYSTEM,
        destination = its.REFUNDED
        ) )


    add( workflow.Transition(
        transition_id = 'ship',
        title=_(u'Ship'),
        source = its.NEW,
        trigger = iworkflow.SYSTEM,
        destination = its.SHIPPED
        ) )
    
    return transitions

class ItemWorkflow( workflow.Workflow ):

    def __init__( self ):
        super( ItemWorkflow, self).__init__( create_item_workflow())

ItemWorkflowAdapter = workflow.AdaptedWorkflow( ItemWorkflow() )

if __name__ == '__main__':
    wf = ItemWorkflow()
    print wf.toDot()


    



"""
minimal zope3 style store
"""

from zope import interface
from zope.app.component import site

import interfaces

class Store( site.SiteManagerContainer ):
    
    interface.implements( interfaces.IStore )

"""
minimal zope3 style store
"""

from zope import interface
from zope.app.component import site

import interfaces

class Store( site.SiteManagerContainer ):
    
    interface.implements( interfaces.IStore )




# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""

$Id: order.py 1488 2008-04-23 22:29:06Z horacio.duran $
"""

from zope import interface

from hurry.workflow import interfaces as iworkflow
from hurry.workflow import workflow

from getpaid.core.interfaces import workflow_states, IOrder, IPaymentProcessor, IDefaultFinanceWorkflow
from zope import component
from zope.i18nmessageid import MessageFactory
_ = MessageFactory('getpaid')

def create_fulfillment_workflow( ):

    fs = workflow_states.order.fulfillment

    transitions = []
    add = transitions.append

    add( workflow.Transition( transition_id = 'create',
                              title= _(u'Create'),
                              source = None,
                              destination = fs.NEW ) )

    # needs condition on charged finance state..
    add( workflow.Transition( transition_id = 'process-order',
                              title = _(u'Process Order'),
                              source = fs.NEW,
                              destination = fs.PROCESSING ) )

    add( workflow.Transition( transition_id = 'deliver-processing-order',
                              title = _(u'Order Processed'),
                              #trigger = iworkflow.SYSTEM,
                              source = fs.PROCESSING,
                              destination = fs.DELIVERED ) )
    
    add( workflow.Transition( transition_id = 'cancel-order',
                              title = _(u'Will Not Deliver'),
                              destination = fs.WILL_NOT_DELIVER,
                              source = fs.PROCESSING ) )

    add( workflow.Transition( transition_id = 'cancel-new-order',
                              title = _(u'Will Not Deliver'),
                              source = fs.NEW,
                              destination = fs.WILL_NOT_DELIVER
                              ) )

    return transitions

#Separate workflow for shippable orders, I has different names on some of the 
#states and also a few states more
def create_shippable_fulfillment_workflow( ):

    fs = workflow_states.shippable_order.fulfillment

    transitions = []
    add = transitions.append

    add( workflow.Transition( transition_id = 'create',
                              title= _(u'Create'),
                              source = None,
                              destination = fs.NEW ) )

    # needs condition on charged finance state..
    add( workflow.Transition( transition_id = 'process-order',
                              title = _(u'Process Order'),
                              source = fs.NEW,
                              destination = fs.PROCESSING ) )

    add( workflow.Transition( transition_id = 'deliver-processing-order',
                              title = _(u'Order Processed'),
                              #trigger = iworkflow.SYSTEM,
                              source = fs.PROCESSING,
                              destination = fs.DELIVERED ) )
    
    add( workflow.Transition( transition_id = 'cancel-order',
                              title = _(u'Will Not Deliver'),
                              destination = fs.WILL_NOT_DELIVER,
                              source = fs.PROCESSING ) )

    add( workflow.Transition( transition_id = 'cancel-new-order',
                              title = _(u'Will Not Deliver'),
                              source = fs.NEW,
                              destination = fs.WILL_NOT_DELIVER
                              ) )

    return transitions


def create_finance_workflow( ):

    fs = workflow_states.order.finance

    transitions = []
    add = transitions.append

    # REVIEWING
    add( workflow.Transition( transition_id = 'create',
                              title = _(u'Create'),
                              source = None,
                              destination = fs.REVIEWING ) )


    add( workflow.Transition( transition_id = 'reviewing-declined',
                              title=_(u'Payment Declined'),
                              trigger = iworkflow.SYSTEM,
                              source = fs.REVIEWING,
                              destination = fs.PAYMENT_DECLINED ) )


    add( workflow.Transition( transition_id = 'authorize',
                              title = _(u'Authorize'),
                              trigger = iworkflow.SYSTEM,
                              source = fs.REVIEWING,
                              destination = fs.CHARGEABLE ) )

    # CHARGEABLE TRANSITIONS
    add( workflow.Transition( transition_id = 'charge-chargeable',
                              title = _(u'Charge Order'),
                              source = fs.CHARGEABLE,
                              destination = fs.CHARGING,
                              trigger = iworkflow.AUTOMATIC ) )

    add( workflow.Transition( transition_id = 'cancel-chargeable',
                              title = _(u'Cancel Order'),
                              source = fs.CHARGEABLE,
                              destination = fs.CANCELLED ) )
                              
    # CHARGING TRANSITIONS
    add( workflow.Transition( transition_id = 'decline-charging',
                              title = _(u'Processor Declined'),
                              source = fs.CHARGING,
                              destination = fs.PAYMENT_DECLINED,
                              trigger = iworkflow.SYSTEM ) )
                              
    add( workflow.Transition( transition_id = 'charge-charging',
                              title = _(u'Processor Charged'),
                              source = fs.CHARGING,
                              destination = fs.CHARGED,
                              trigger = iworkflow.SYSTEM ) )
                                                    

    # add( workflow.Transition( transition_id = 'authorize-chargeable',
    #                           title = _(u'Authorize Order'),
    #                           source = fs.CHARGEABLE,
    #                           destination = fs.CHARGEABLE ) )

    # CHARGED TRANSITIONS
    # add( workflow.Transition( transition_id = 'authorize-charged',
    #                           title = _(u'Authorize Order'),
    #                           source = fs.CHARGED,
    #                           destination = fs.CHARGED ) )

    # add( workflow.Transition( transition_id = 'refund-order',
    #                           title = _(u'Refund Order'),
    #                           source = fs.CHARGED,
    #                           destination = fs.REFUNDED ) )

    # add( workflow.Transition( transition_id = 'charge-charged',
    #                           title = _(u'Charge Order'),
    #                           source = fs.CHARGED,
    #                           destination = fs.CHARGED ) )


    # PAYMENT DECLINED TRANSITIONS

    add( workflow.Transition( transition_id = 'cancel-declined',
                              title=_(u'Cancel Order'),
                              source = fs.PAYMENT_DECLINED,
                              destination = fs.CANCELLED ) )

    # SYSTEM TRANSITIONS
    
    add( workflow.Transition( transition_id = 'processor-cancelled',
                              title = _(u'Processor Cancel'),
                              source = fs.REVIEWING,
                              destination = fs.CANCELLED_BY_PROCESSOR,
                              trigger = iworkflow.SYSTEM, ) )

    return transitions



class FulfillmentWorkflow( workflow.Workflow ):
    interface.implements( iworkflow.IWorkflow )
    def __init__( self ):
        super( FulfillmentWorkflow, self).__init__( create_fulfillment_workflow())

class ShippableFulfillmentWorkflow( workflow.Workflow ):
    interface.implements( iworkflow.IWorkflow )
    def __init__( self ):
        super( ShippableFulfillmentWorkflow, self).__init__( create_shippable_fulfillment_workflow())

class FinanceWorkflow( workflow.Workflow ):
    interface.implements( iworkflow.IWorkflow )
    def __init__( self ):
        super( FinanceWorkflow, self).__init__( create_finance_workflow() )

# if we passed in IOrder we could have all of these registered for us, but
# to make it easier to change these, we plugin them in via zcml.
FulfillmentWorkflowAdapter, FulfillmentState, FulfillmentInfo = workflow.ParallelWorkflow(
    workflow.AdaptedWorkflow( FulfillmentWorkflow() ),
    workflow_states.order.fulfillment.name,
    )

ShippableFulfillmentWorkflowAdapter, ShippableFulfillmentState, ShippableFulfillmentInfo = workflow.ParallelWorkflow(
    workflow.AdaptedWorkflow( ShippableFulfillmentWorkflow() ),
    workflow_states.order.fulfillment.name,
    )


FinanceWorkflowAdapter, FinanceState, FinanceInfo = workflow.ParallelWorkflow(
    workflow.AdaptedWorkflow( FinanceWorkflow() ),
    workflow_states.order.finance.name,
    )

interface.classImplements( FinanceWorkflowAdapter, IDefaultFinanceWorkflow )


if __name__ == '__main__':
    wk = FinanceWorkflow(None)





# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""

note we should potentially expand this workflow to show typical elements from the physical shipment
workflow, waiting for pickup, delivery.. hmm.. actually better is just have a shipment status
field, which can be dynamically looked up via the shipping provider.

 transition : create
    state - new

 transition : charge 
    state - charging
    condition - (*)

 transition : charged
    source - chargig
    state - charged
    trigger - system

 transition : charge-declined
    state - 
 
 transition : ship-new
    state - shipped 
    condition - (*)
   
 transition : ship-charged (*)
    state - 
   shipped

 delivered

 (*) the viability of these transitions (conditions), depends on store settings for how to
     capture an order. ie. in a charge before ship model, shipments are first charged, then
     shipped, another might charge the entire order before fulfillment.

$Id: shipment.py 1141 2007-12-30 23:47:15Z kapilt $
"""

from zope.interface import implements

from hurry.workflow import interfaces as iworkflow
from hurry.workflow import workflow

from getpaid.core.interfaces import workflow_states
from zope.i18nmessageid import MessageFactory
_ = MessageFactory('getpaid')

def CheckAuthorized( wf, context ):
    return True

def CheckCharged( wf, context ):
    return True

def create_shippment_workflow( ):

    ss = workflow_states.shipment

    transitions = []
    add = transitions.append
    
    add( workflow.Transition(
        transition_id = 'create',
        title=_(u'Create'),
        source = None,
        destination = ss.NEW
        ) )

    add( workflow.Transition(
        transition_id = 'charge',
        title=_(u"Charge"),
        source = ss.NEW,
        destination = ss.CHARGING
        ) )

    add( workflow.Transition(
        transition_id = 'declined',
        title = _(u'Declined'),
        source = ss.CHARGING,
        destination = ss.DECLINED,
        trigger = iworkflow.SYSTEM
        ) )
        
    add( workflow.Transition(
        transition_id = 'charge',
        title = _(u'Charged'),
        source = ss.CHARGING,
        destination = ss.CHARGED,
        trigger = iworkflow.SYSTEM
        ) )

    add( workflow.Transition(
        transition_id = 'already-charged-order',
        title = _(u'Order Charged'),
        condition = CheckCharged,
        trigger = iworkflow.AUTOMATIC,
        source = ss.NEW,
        destination = ss.CHARGED
        ) )

    add( workflow.Transition(
        transition_id = 'ship-charged',
        title = _(u'Ship'),
        source = ss.CHARGED,
        destination = ss.SHIPPED,
        ) )    

    add( workflow.Transition(
        transition_id = 'delivered',
        title = _(u'Delivered'),
        source = ss.SHIPPED,
        destination = ss.DELIVERED,
        trigger = iworkflow.SYSTEM
        ) )

    return transitions

class ShipmentWorkflow( workflow.Workflow ):

    def __init__( self ):
        super( ShipmentWorkflow, self).__init__( create_shippment_workflow() )

ShipmentWorkflowAdapter = workflow.AdaptedWorkflow( ShipmentWorkflow() )

if __name__ == '__main__':
    wf = ShipmentWorkflow()
    print wf.toDot()




# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

import unittest, datetime, doctest

from getpaid.core.tests import base
from getpaid.core import order
from getpaid.core.interfaces import IOrderManager, IOrderWorkflowLog, IOrder, workflow_states

from zope.app.testing import placelesssetup, ztapi
from zope.testing.doctestunit import DocFileSuite

from zope import component 

class OrderLogTests( base.GetPaidTestCase ):
    
    def setUp( self ):
        super(OrderLogTests, self).setUp()
        
    def testLog( self ):
        self.orders = list( base.createOrders() )
        order1 = self.orders[0]
        order1.finance_workflow.fireTransition('create')

        olog = IOrderWorkflowLog( order1 )
        history = list( iter( olog ) )
        self.assertEqual( len( history ), 1 )
        entry = history[0]
        self.assertEqual( entry.transition, u'Create')
        self.assertEqual( entry.new_state, workflow_states.order.finance.REVIEWING)
    
class OrderQueryTests( base.GetPaidTestCase ):

    def setUp( self ):
        super( OrderQueryTests, self).setUp()        
        self.orders = list( base.createOrders() )
        self.manager = component.getUtility( IOrderManager )
        
    def tearDown( self ):
        super( OrderQueryTests, self).tearDown()
        self.orders = None
        self.manager = None
        
    def testDateQuery( self ):
        self.orders[0].creation_date = datetime.datetime.now() - datetime.timedelta( 30 )
        self.manager.storage.reindex( self.orders[0] )
        
        self.orders[1].creation_date = datetime.datetime.now() - datetime.timedelta( 120 )
        self.manager.storage.reindex( self.orders[1] )

        # find orders in the last week
        results = order.query.search( creation_date = datetime.timedelta(7)  )
        self.assertEqual( len(results), 7 )
        
        now = datetime.datetime.now()
        start_date = now - datetime.timedelta( 125 )
        end_date = now - datetime.timedelta( 7 )
        results = order.query.search( creation_date = ( start_date, end_date ) )
        self.assertEqual( len(results), 2 )

        
    def testStateQuery( self ):
        self.orders[0].finance_workflow.fireTransition('create')
        self.orders[0].finance_workflow.fireTransition('authorize')

        self.manager.storage.reindex( self.orders[0] )        

        self.orders[1].finance_workflow.fireTransition('create')        
        self.orders[1].finance_workflow.fireTransition('authorize')
        self.orders[1].finance_workflow.fireTransition('cancel-chargeable')        
        self.manager.storage.reindex( self.orders[1] )

        self.orders[2].finance_workflow.fireTransition('create')        
        self.orders[2].finance_workflow.fireTransition('authorize')
        self.orders[2].finance_workflow.fireTransition('charge-chargeable') 
        self.manager.storage.reindex( self.orders[2] )
        
        self.assertEqual( len( order.query.search( finance_state = workflow_states.order.finance.CHARGEABLE ) ), 1 )
        self.assertEqual( len( order.query.search( finance_state = workflow_states.order.finance.CANCELLED ) ), 1 )
        self.assertEqual( len( order.query.search( finance_state = workflow_states.order.finance.CHARGING ) ), 1 )                

    def testCombinedQuery( self ):
        self.orders[0].finance_workflow.fireTransition('create')
        self.orders[0].finance_workflow.fireTransition('authorize')
        self.orders[0].creation_date = created = datetime.datetime.now() - datetime.timedelta( 30 )
        self.manager.storage.reindex( self.orders[0] )
        self.assertEqual( len( order.query.search( finance_state = workflow_states.order.finance.CHARGEABLE,
                                                   creation_date = ( created - datetime.timedelta(1),
                                                                     created + datetime.timedelta(1) )
                                                   )),
                          1
                          )
                          
def test_suite():
    return unittest.TestSuite((
        DocFileSuite('../order.txt',
                     setUp=base.coreSetUp,
                     tearDown=placelesssetup.tearDown,
                     optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
                     ),    
        unittest.makeSuite( OrderQueryTests ),
        unittest.makeSuite( OrderLogTests ),        
        ))                          



# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""

very lightweight catalog implementation

"""


import datetime

from persistent.dict import PersistentDict
from zope.app.container.btree import BTreeContainer
from zope.app.intids.interfaces import IIntIds

from BTrees.IFBTree import weightedIntersection, intersection

class ResultSet:
    """Lazily accessed set of objects."""

    def __init__(self, uids, storage):
        self.uids = uids or []
        self.storage = storage

    def __len__(self):
        return len(self.uids)

    def __iter__(self):
        for uid in self.uids:
            yield self.storage[ str( uid ) ]

class IndexedRecords( BTreeContainer ):

    # define something like this for indexing and querying
    # 
    #index_map = { 'index_name' : ( Factory, query_method )}


    
    def __init__( self ):
        super( IndexedRecords, self).__init__()
        self.indexes = PersistentDict()
        self.setupIndexes()
        
    def setupCatalog( self):
        for index_name, (factory, query) in self.index_map:
            self.indexes[ index_name ] = factory
        
    def query( self, **args ):
        results = self.apply( args )        
        return ResultSet( results, self )

    def apply(self, query):
        results = []
        for index_name, index_query in query.items():
            index = self.indexes[index_name]
            r = index.apply(index_query)
            if r is None:
                continue
            if not r:
                # empty results
                return r
            results.append((len(r), r))

        if not results:
            # no applicable indexes, so catalog was not applicable
            return None

        results.sort() # order from smallest to largest

        _, result = results.pop(0)
        for _, r in results:
            _, result = weightedIntersection(result, r)

        return result
    
    def __setitem__( self, key, object):
        super( IndexedRecords, self ).__setitem__( key, object )
        self.index( key, object )

    def reset_index( self ):
        # reindex all orders
        for index in self.indexes.values():
            index.clear()
        for order in self.values():
            self.index( order )

    def reindex( self, key ):
        self.unindex( key )
        object = self[ key ]
        self.index( key, object )
            
    def index( self, key, object ):
        doc_id = int( key )
        for attr, index in self.indexes.items():
            value = getattr( object, attr, None)
            if callable( value ):
                value = value()
            if value is None:
                continue
            index.index_doc( doc_id, value )

    def unindex( self, key ):
        for index in self.indexes.values():
            index.unindex_doc( int( key ) )
        
    def __delitem__( self, key ):
        super( IndexedRecords, self).__delitem__( key )
        self.unindex( key )

    #################################
    # junk for z2.9 / f 1.4
    def manage_fixupOwnershipAfterAdd(self, *args):
        return

    def manage_setLocalRoles( self, *args ):
        return 

class RecordQuery( object ):
    """
    simple query construction.. it might be problematic for other storages without collapsing
    sort clauses where possible. best to minimize any query combinations in the released product.

    main interface to searching is the search method

    from getpaid.core.order import query
    from datetime import timedelta
    
    # find orders from the last week
    results = query.search( creation_date = timedelta(7) )

    """
    
    indexes =  [ 'finance_state',
                 'fulfillment_state',
                 'user_id',
                 'creation_date' ]
    default_sort = None
    @classmethod
    def search( cls, data=None, **kw ):
        """ take a dictionary of key, value pairs, and based on available queries/indexes
        construct query and return results """
        
        results = None
        if data is None:
            data = kw
        elif data and kw:
            data.update( kw )

        storage = cls.getStorage()
        indexes = [(name, query) for name, (f, query) in storage.index_map.items()]
        
        for term, query_method in indexes:
            term_value = data.get( term )
            if term_value is None:
                continue
            
            term_results = getattr( cls, query_method )( term, term_value )
            if term_results is None: # short circuit .. default and intersection
                return []
            if results is None:
                results = term_results
            else:
                results = cls.merge( results, term_results )

        # actualize to order objects
        results = cls.generate( results )

        if 'no_sort' in kw:
            return results        
        elif not storage.default_sort:
            return results
        return cls.sort( results, *storage.default_sort)
        
    @classmethod
    def generate( cls, results ):
        """ used to actualize results from ifsets to 
        """
        intids = cls.getIntIds()
        return ResultSet( results, intitds )

    @classmethod
    def getIntIds( self ):
        return component.getUtility( IIntIds )

    @staticmethod
    def sort( results, attribute, reverse=False ):
        results = list( results )
        results.sort( lambda x,y: cmp( getattr( x, attribute ), getattr( y, attribute ) ) )
        if reverse:
            results.reverse()
        return results

    @staticmethod
    def merge( *results  ):
        return reduce( intersection, [res for res in results if res is not None] )        
        
    @classmethod
    def date( cls, field_name, delta = None ):
        """ query by creation date, pass in either a delta to be used from the current time
            or a tuple of start date, end date to return orders from.
        """
        if not delta:  # default to one last week ?
            delta = datetime.timedelta(7)
        if isinstance( delta, tuple ):
            value = delta
        else:
            now = datetime.datetime.now()
            value = ( now-delta, now )
        storage = cls.getStorage()
        return storage.apply( { field_name : value } )
        

    @classmethod
    def field( cls, field_name, value ):
        storage = cls.getStorage()
        return storage.apply( { field_name : (value, value) } )
        

    @classmethod
    def getStorage( cls ):
        raise NotImplemented



# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""
$Id: options.py 1516 2008-05-03 12:57:53Z cwithers $
"""
try:
    from zope.annotation import IAnnotations
except ImportError:
    # BBB for Zope 2.9
    from zope.app.annotation import IAnnotations
from persistent.dict import PersistentDict
from persistent import Persistent
from zope import schema
from zope.interface import classImplements, implements

import interfaces

_marker = object()

class PropertyBag( object ):
    """ you can subclass and init to a particular schema, meant for transient adapter bags
    """
    def __init__( self, **kw ):
        for field_name, field in schema.getFieldsInOrder( self.schema ):
            if field_name in kw:
                field.set( self, kw[ field_name ] )

    @classmethod
    def initclass( cls, iface ):
        cls.schema = iface
        for field_name, field in schema.getFieldsInOrder( iface ):
            setattr( cls, field_name, field.default )
        classImplements( cls, iface )            

    @classmethod
    def makeclass( cls, schema ):
        klass = type( "transientbag", ( cls, ), {} )
        klass.initclass( schema )
        classImplements( klass, schema )
        return klass

    @classmethod
    def makeinstance( cls, schema ):
        return cls.makeclass( schema )()
        
    # post initialziation class method
    @classmethod
    def frominstance( cls, instance):
        assert cls.schema.providedBy( instance )
        d = {}
        for field_name, field in schema.getFieldsInOrder( cls.schema ):
            d[field_name] = field.get(instance)
        return cls( **d )
        
class PersistentBag( Persistent, PropertyBag):

    def getProperty( self, property_name ):
        return getattr( self, property_name, None )

    def setProperty( self, property_name, property_value ):
        setattr( self, property_name, property_value )
        self._p_changed = True
        
class PersistentOptions( object ):

    implements( interfaces.IPersistentOptions )
    
    _storage = None
    
    def __init__( self, context ):
        self.context = context

    def storage( self, name=None ):
        """ name if given is the key of a persistent dictionary off of
        the annotation.
        """ 
        if self._storage is None:
            annotations = IAnnotations( self.context )
            self._storage = annotations.get( self.annotation_key, None )
            if self._storage is None:
                annotations[ self.annotation_key ] = self._storage = PersistentDict()

        if name is None:
            return self._storage
        if name in self._storage:
            return self._storage[name]
        
        self._storage[ name ] = PersistentDict()
        return self._storage[name]

    def getProperty( self, property_name ):
        return self.storage().get( property_name )

    def getFieldProperty( self, field ):
        value = self.storage().get( field.__name__, _marker )
        if value is _marker:
            return field.default
        return value
    
    def setProperty( self, property_name, property_value ):
        self.storage()[ property_name ] = property_value

    def nullProperty( self, *args):
        return None

    def wire( cls, name, key, *interfaces, **options ):
        fields = {}
        bases = (cls, ) + options.get('bases', ())
        
        for iface in interfaces:
            for field in schema.getFields( iface ).values():
                fields[ field.__name__ ] = property( lambda self, field=field: self.getFieldProperty( field ),
                lambda self, value, field_name=field.__name__: self.setProperty( field_name, value ) )
        new_class = type( name, bases, fields)
        cls.annotation_key = key
        classImplements( new_class, interfaces )
        return new_class

    wire = classmethod( wire )
    
class FormSchemas:
    # Abstract Base Class!

    implements( interfaces.IFormSchemas )

    interfaces = {}
    bags = {}
    
    def getInterface(self,section):
        return self.interfaces[section]

    def getBagClass(self,section):
        return self.bags[section]
    



# Copyright (c) 2008 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

from zope import interface, component

from getpaid.core.interfaces import IStoreSettings, IShippableLineItem, IOrder
from getpaid.core.payment import Address, ContactInformation

import interfaces

class OriginRouter( object ):

    component.adapts( IOrder )
    interface.implements( interfaces.IOriginRouter )
    
    def __init__( self, context ):
        self.context = context
            
    def getStoreSettings( self ):
        store_settings = component.getUtility( IStoreSettings )
        contact = ContactInformation( name = ( store_settings.contact_company or store_settings.store_name ),
                                      phone_number = store_settings.contact_phone,
                                      email = store_settings.contact_email )
                                      
        address = Address( first_line = store_settings.contact_address,
                           second_line = store_settings.contact_address2,
                           city = store_settings.contact_city,
                           state = store_settings.contact_state,
                           postal_code = store_settings.contact_postalcode,
                           country = store_settings.contact_country )
        
        return contact, address
        
    getOrigin = getStoreSettings


# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""
$Id: cart.py 1611 2008-06-06 20:39:20Z calvinhp $
"""

import decimal

from persistent import Persistent
from zope import component
from zope.interface import implements

from zope.app.container.ordered import OrderedContainer

import interfaces

class ShoppingCart( OrderedContainer ):
    """
    A shopping cart
    """
    implements( interfaces.IShoppingCart )

    last_item = None
    
    def size( self ):
        return sum(i.quantity for i in self.values())

    def __setitem__( self, key, value ):
        super(ShoppingCart, self).__setitem__( key, value)
        self.last_item = key
        
    def __delitem__( self, key ):
        if not key in self:
            return
        super( ShoppingCart, self).__delitem__( key )
        if self.last_item == key:
            if len(self)>0:
                self.last_item = self.keys()[-1]
            else:
                self.last_item = None

class CartItemTotals( object ):

    implements( interfaces.ILineContainerTotals )
    
    def __init__( self, context ):
        self.shopping_cart = context

    def getTotalPrice( self ):
        if not self.shopping_cart:
            return 0
        
        total = 0
        total += self.getSubTotalPrice()
        total += self.getShippingCost()
        for tax in self.getTaxCost():
            total += tax['value']
        
        return float( str( total ) )            

    def getSubTotalPrice( self ):
        if not self.shopping_cart:
            return 0
        total = 0
        for item in self.shopping_cart.values():
            d = decimal.Decimal ( str(item.cost ) ) * item.quantity
            total += d        
        return total
        
    def getShippingCost( self ):
        if not interfaces.IShippableOrder.providedBy( self ):
            return 0
        return decimal.Decimal( str( self.shipping_cost ) )

    def getTaxCost( self ):
        """ get the list of dictionaries containing the tax info """
        tax_utility = component.getUtility( interfaces.ITaxUtility )
        return tax_utility.getTaxes( self )




# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""
$Id: event.py 1371 2008-04-03 08:09:56Z kapilt $
"""

from zope.interface import implements
from zope.app.event.objectevent import ObjectEvent
from zope.event import notify

import interfaces

class PayableCreationEvent( ObjectEvent ):

    implements( interfaces.IPayableCreationEvent )
    
    def __init__( self, object, payable, payable_interface ):
        super( PayableCreationEvent, self).__init__( object )
        self.payable = payable
        self.payable_interface = payable_interface

class BeforeCheckoutEvent( ObjectEvent ):
    
    def __init__( self, context, order, request):
        super( BeforeCheckoutEvent, self).__init__( order )
        self.request = request
        self.context = context

# we need to move the payable markers into getpaid
# def objectCreation( object, event ):
#     
#     if interfaces.IPayable.providedBy( object ):
#         notify( PayableCreationEvent( object, object, interfaces.IPayable ) )



# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.



from zope import interface

import csv, StringIO
import interfaces


def getOrderCreationDate( order ):
    pass
    
def getOrderTotalPrice( order ):
    return order.getTotalPrice()

class Attribute( object ):    
    def __init__(self, name):
        self.name = name
    def __call__( self, order ):
        return getattr( order, self.name, '')
    
def defaults( field_accessor_map, field_order ):
    for fn in field_order:
        if fn not in field_accessor_map:
            field_accessor_map[fn]=Attribute(fn)
    
class AttributeCSVSetReport( object ):

    interface.implements( interfaces.IOrderSetReport )
    
    mime_type = 'text/csv'

    # the order fields appear in a csv report
    field_order = ["order_id", "user_id", "finance_state", "fulfillment_state", "price", "creation_date"]
    
    field_title_map = dict(
        order_id = "Order Id",
        user_id = "Customer Id",
        finance_state = "Status",
        fulfillment_state = "Fulfillment",
        creation_date = "Created",
        price = "Price"
        )

    field_accessor_map = dict( 
        creation_date = getOrderCreationDate,
        price = getOrderTotalPrice
        )
        
    defaults( field_accessor_map, field_order )
    
    def __init__( self, context ):
        self.context = context
        
    def sortOrders( self, orders ):
        return orders

    def writeRows( self, writer, orders ):
        field_getters = []
        for field_name in self.field_order:
            field_getters.append( self.field_accessor_map.get( field_name ) )
            
        for order in orders:
            writer.writerow( [getter( order ) for getter in field_getters ] )
                
    def __call__( self, orders ):
        
        orders = self.sortOrders( orders )
        
        io = StringIO.StringIO()
        writer = csv.writer( io )
        writer.writerow( [ self.field_title_map.get( field_name, field_name) \
                           for field_name in self.field_order ] )
        self.writeRows( self, writer, orders )

        return io.getvalue()

class InventoryCSVReport( AttributeCSVSetReport ):

    field_order = ["product_id", "title", "quantity", "average_unit_price", "price"]
    
    field_accessor_map = dict(
      product_id = lambda x: x['product_id'],
      title = lambda x: x['title'],
      quantity = lambda x: x['quantity'],
      average_unit_price = lambda x: x['price']/float(x['quantity']),
      price = lambda x: x['price']
    )
    
    def sortOrders( self, orders ):
        items = {}
        for o in orders:
            for i in o.shopping_cart.values():
                item_info = items.get( i.item_id )
                if item_info is None:
                    items[ i.item_id ] = item_info = dict( product_id = i.item_id, title = i.title, quantity=0, price=0)
                item_info[ 'quantity'] += i.quantity
                item_info[ 'price' ] += i.quantity * i.price
            
        product_ids = items.keys()
        product_ids.sort()
        return [ items[product_id] for product_id in product_ids ]
    

class FulfillmentCSVReport( AttributeCSVSetReport ):
    
    # the order fields appear in a csv report
    field_order = ["order_id", "name", 'phone', 'email', 'title', 'quantity', 'price']
    
    field_title_map = t = dict( AttributeCSVSetReport.field_title_map )

    field_title_map = dict(
        order_id = "Order Id",
        user_id = "Customer Id",
        finance_state = "Status",
        fulfillment_state = "Fulfillment",
        creation_date = "Created",
        price = "Price"
        )

    field_accessor_map = dict( 
        creation_date = getOrderCreationDate,
        price = getOrderTotalPrice
        )
        



# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""

workflow event driven payment processor integration and property bags needed for
an order.
"""

from getpaid.core import interfaces, options 
from zope import component, interface

class Address( options.PersistentBag ): pass
Address.initclass( interfaces.IAddress  )    
    
class ShippingAddress( options.PersistentBag ): pass
ShippingAddress.initclass( interfaces.IShippingAddress )    
 
class BillingAddress( options.PersistentBag ): pass
BillingAddress.initclass( interfaces.IBillingAddress )    
    
class ContactInformation( options.PersistentBag ): pass
ContactInformation.initclass( interfaces.IUserContactInformation )    
    
def fireAutomaticTransitions( order, event ):    
    """ fire automatic transitions for a new state """ 
    order.finance_workflow.fireAutomatic()


def processorWorkflowSubscriber( order, event ):
    """
    fire off transition from charging to charged or declined based on
    payment processor interaction.
    """

    # check for a payment processor associated with the 
    # there is a default notion here that the workflows for finance / fulfillment can't share state names
    # 
    if order.finance_state == event.destination:
        adapter = component.queryMultiAdapter( (order, order.finance_workflow.workflow() ),
                                               interfaces.IWorkflowPaymentProcessorIntegration )
                                               
    elif order.fulfillment_state == event.destination:
        adapter = component.queryMultiAdapter( (order, order.fulfillment_workflow.workflow() ),
                                               interfaces.IWorkflowPaymentProcessorIntegration )
    else:
        return
                                              
    if adapter is None:
        return

    return adapter( event )

class DefaultFinanceProcessorIntegration( object ):
    
    interface.implements( interfaces.IWorkflowPaymentProcessorIntegration )
    
    def __init__( self, order, workflow):
        self.order = order
        self.workflow = workflow
        
    def __call__( self, event ):
        if event.destination != interfaces.workflow_states.order.finance.CHARGING:
            return

        # on orders without any cost, forgo invoking the payment processor
        price = self.order.getTotalPrice()
        if not price > 0:
            return

        # ick.. get a hold of the store
        # this is a little gross, we need some access to context, so we fetch line items
        # till we find something that resolves to an object, and try to get the store from that
        # 
        context = component.queryUtility( interfaces.IStore )
        if context is None:
            from Products.CMFCore.utils import getToolByName
            ob = None
            for i in self.order.shopping_cart.values():
                if interfaces.IPayableLineItem.providedBy( i ):
                    ob = i.resolve()
            if ob is None:
                raise AttributeError("can't get store, TODO - please switch processors settings to utility adapters")
            context = getToolByName( ob, 'portal_url').getPortalObject()

        processor = component.getAdapter( context,
                                          interfaces.IPaymentProcessor,
                                          self.order.processor_id )

        result = processor.capture( self.order, self.order.getTotalPrice() )
    
        if result == interfaces.keys.results_async:
            return
        elif result == interfaces.keys.results_success:
            self.order.finance_workflow.fireTransition('charge-charging')
        else:
            self.order.finance_workflow.fireTransition('decline-charging', comment=result)


CREDIT_CARD_TYPES = ( u"Visa", u"MasterCard", u"Discover", u"American Express" )

class CreditCardTypeEnumerator(object):
    interface.implements(interfaces.ICreditCardTypeEnumerator)

    def __init__(self, context):
        self.context = context

    def acceptedCreditCardTypes(self):
        return CREDIT_CARD_TYPES

    def allCreditCardTypes(self):
        return CREDIT_CARD_TYPES



# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""

$Id: item.py 1445 2008-04-17 18:56:54Z kapilt $
"""

from persistent import Persistent

from zope.interface import implements
from zope import component

from zope.app.container.interfaces import ILocation
try:
    from zope.annotation.interfaces import IAttributeAnnotatable
except ImportError:
    # BBB for Zope 2.9
    from zope.app.annotation.interfaces import IAttributeAnnotatable
from zope.app.intid.interfaces import IIntIds

from hurry.workflow.interfaces import IWorkflowState, IWorkflowInfo
from getpaid.core import interfaces


class LineItem( Persistent ):
    """
    an item in the cart

    lineitems are not generically attribute annotatable, which typically requires
    zodb persistence, instead to enable storage in other mediums, we use a specific
    limited set of components that use annotations on line items, specifically the
    workflow engine to enable fulfillment workflows on individual items.
    """
    implements( interfaces.ILineItem, IAttributeAnnotatable )

    
    # default attribute values, item_id is required and has no default
    name = ""
    description = ""
    quantity = 0
    cost = 0
    product_code = ""

    @property
    def fulfillment_state( self ):
        return IWorkflowState( self ).getState()

    @property
    def fulfillment_workflow( self ):
        return IWorkflowInfo( self )

    def clone( self ):
        clone = self.__class__.__new__( self.__class__ )
        clone.__setstate__( self.__getstate__() )
        if ILocation.providedBy( clone ):
            del clone.__name__, clone.__parent__
        return clone

class ShippableLineItem( LineItem ):
    
    implements( interfaces.IShippableLineItem )
    weight = ""
    um_weight = ""
    um_distance = ""
    length = ""
    height = ""
    width = ""

class PayableLineItem( LineItem ):
    """
    an item in the cart for a payable
    """
    implements( interfaces.IPayableLineItem )

    # required
    uid = None
    
    def resolve( self ):
        utility = component.getUtility( IIntIds )
        return utility.queryObject( self.uid )
        
class PayableShippableLineItem( ShippableLineItem, PayableLineItem ):
    """
    a shippable item in a cart for a payable
    """



import random, string, sys

from zope import interface
from zope.app.intid.interfaces import IIntIds

from getpaid.core import cart, order, payment, item, interfaces

class Product( object ):

    interface.implements( interfaces.IPayable )

class ProductGenerator( object ):

    _products = {}

    @classmethod
    def find( cls, uid ):
        return cls._products.get( uid )

    def __setitem__( self, k, v ):
        self._products[k] = v

    @classmethod
    def keys( self ):
        return self._products.keys()

    @property
    def uid( self ):
        while 1:
            uid = random.randint( 1, 1000 )
            if uid in self._products:
                continue
            break
        return str(uid)

    @property
    def supplier( self ):
        return unicode( random.choice( ['twax', 'bwax', 'cwax'] ) )

    @property
    def product_code( self ):
        return ''.join( random.sample( string.letters, 5 ) )
    
    @property
    def price( self ):
        return ( random.random()*10 )
    
    def new( self ):
        uid = self.uid
        p = Product()
        
        p.made_payable_by = self.supplier
        p.product_code = self.product_code
        p.price = self.price
        p.uid = uid
        self[ uid ] = p
        return p
    
class ProductIntId( object ):

    interface.implements( IIntIds )

    def queryId( self, ob ):
        return ob.uid
    
class Item( item.PayableShippableLineItem ):

    def resolve( self ):
        return ProductGenerator.find( self.uid )

class Mock( object ): pass

class OrderGenerator( object ):

    def __call__( self ):
        _order = order.Order()
        _order.billing_address = self.bill_address
        _order.shipping_address = self.ship_address
        _order.shopping_cart = self.shopping_cart
        _order.contact_information = self.contact_information
        _order.setOrderId( self.order_id )
        
        return _order

    @property
    def order_id( self ):
        return random.randint( 1, sys.maxint )

    @property
    def shopping_cart( self ):
        shopping_cart = cart.ShoppingCart()
        for i in range( random.randint( 2, 12 ) ):
            item = Item()
            item.uid = random.choice( ProductGenerator.keys() )
            item.quantity = random.randint( 1, 5 )

            if item.uid in shopping_cart:
                shopping_cart[ item.uid ].quantity += item.quantity
                continue
            
            item.name = random.choice( (u"Laser", u"Book", u"Rabbit", u"Snake", u"Lizard") )
            item.description = u''.join( random.sample( string.letters, 30) )
            item.weight = random.random()*3.0
            item.product_code = ''.join( random.sample( string.letters, 8) )
            item.cost = random.randint( 1, 100 )
            shopping_cart[ item.uid ] = item
        return shopping_cart
        
    @property
    def contact_information( self ):
        contact = payment.ContactInformation()
        first_name = random.choice( ("Mary", "Mike", "John", "Steve", "Elizabeth" ) )
        last_name  = random.choice( ("Smith", "Burton", "Appleseed", "Xavier" ) )
        phone  = ''.join( random.sample( string.digits, 9 ) )
        contact.name = u"%s %s"%( first_name, last_name )
        contact.phone = phone
        contact.marketing_preference = random.choice( (True, False ) )
        contact.email_html_format = random.choice( (True, False ) )
        return contact

    @property
    def zip( self ):
        return ''.join( random.sample( string.digits, 5 ) )

    @property
    def state( self ):
        return random.choice( ('US-NY', 'US-CA', 'US-VA', 'US-MA' ) )

    @property
    def city( self ):
        return random.choice( (u"York", u"Lancester", u"Shire", u"Burg" ) )
    
    @property
    def address_line( self ):
        numeral = ''.join( random.sample( string.digits, 3) )
        return u"%s Kittyhawk St"%numeral        

    @property
    def ship_address( self ):
        address = payment.ShippingAddress()
        address.ship_first_line  = self.address_line
        address.ship_city = self.city
        address.ship_state = self.state
        address.ship_country = "US"
        address.ship_postal_code = self.zip
        return address
    
    @property
    def bill_address( self ):
        address = payment.BillingAddress()
        address.bill_first_line  = self.address_line
        address.bill_city = self.city
        address.bill_state = self.state
        address.bill_country = "US"
        address.bill_postal_code = self.zip
        return address



def main():
    products = ProductGenerator()
    for i in range( random.randint(20,30) ):
        products.new()

    orders = OrderGenerator()
    orders = OrderGenerator()
    for i in range( 10 ):
        o = orders()
        print o, o.getSubTotalPrice(), len( o.shopping_cart), o.shopping_cart.size()

if __name__ == '__main__':
    main()



# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""
order utility implementation
"""
import datetime, random

from persistent import Persistent
from persistent.list import PersistentList
from persistent.dict import PersistentDict

from BTrees.IFBTree import weightedIntersection, intersection

from zope import component
from zope.interface import implements
from zope.index.field import FieldIndex
from zope.i18nmessageid import MessageFactory
from zope.index.keyword  import KeywordIndex
from zope.schema.fieldproperty import FieldProperty

from zope.app.container.btree import BTreeContainer
from hurry.workflow.interfaces import IWorkflowState, IWorkflowInfo

try:
    from zope.annotation.interfaces import IAttributeAnnotatable
    from zope.annotation.interfaces import IAnnotations
except ImportError:
    # BBB for Zope 2.9
    from zope.app.annotation.interfaces import IAttributeAnnotatable
    from zope.app.annotation.interfaces import IAnnotations

from getpaid.core import interfaces, cart

_ = MessageFactory('getpaid')

try:
    from AccessControl import getSecurityManager
except ImportError:
    getSecurityManager = None

class workflow_state( object ):
    
    def __init__(self, workflow_name ):
        self.workflow_name = workflow_name
        self.__doc__ = "workflow state %s"%workflow_name

    def __get__(self, obj, objtype=None):
        if obj is None:
            return self         
        return component.getAdapter( obj, IWorkflowState, self.workflow_name).getState()            

    def __set__(self, obj, value):
        return component.getAdapter( obj, IWorkflowState, self.workflow_name).setState( value )

    def __delete__(self, obj):
        raise AttributeError, "can't delete attribute"


class Order( Persistent, cart.CartItemTotals ):

    implements( interfaces.IOrder, IAttributeAnnotatable )

    _order_id = None
    shipping_address = None
    billing_address = None
    shopping_cart = None
    processor_order_id = None
    user_id = None
    creation_date = None
    user_payment_info_last4 = None
    user_payment_info_trans_id = None
    name_on_card = None
    bill_phone_number = None

    def __init__( self ):
        self.creation_date = datetime.datetime.now()

    def getOrderId( self ):
        return self._order_id

    def setOrderId( self, order_id):
        if self._order_id is not None:
            raise SyntaxError(_(u"Order Id already set"))
        if not order_id:
            raise TypeError(_(u"Invalid Order Id"))
        self._order_id = order_id

    def setOrderTransId( self, trans_id):
        if self.user_payment_info_trans_id is not None:
            raise SyntaxError(_(u"Transaction Id already set"))
        if not trans_id:
            raise TypeError(_(u"Invalid Transaction Id"))
        self.user_payment_info_trans_id = trans_id

    order_id = property( getOrderId, setOrderId )

    finance_state = workflow_state( "order.finance")

    fulfillment_state = workflow_state( "order.fulfillment")


    def __len__( self ):
        return len(self.shopping_cart)

    @property
    def finance_workflow( self ):
        return component.getAdapter( self, IWorkflowInfo, "order.finance")

    @property
    def fulfillment_workflow( self ):
        return component.getAdapter( self, IWorkflowInfo, "order.fulfillment")


class OrderManager( Persistent ):

    implements( interfaces.IOrderManager )
    
    def __init__( self ):
        self.storage = OrderStorage()

    def store( self, order ):
        self.storage[ order.order_id ] = order
    
    def query( self, **kw ):
        return query.search( **kw )

    def get( self, order_id ):
        return self.storage.get( order_id )

    def newOrderId( self ):
        """
        Return an order id that has some kind of guarantee that it
        hasn't been used for an order before.
        """
        while 1:
            order_id = str( random.randint( 2**10, 2**30 ) )
            if self.get( order_id ) is None:
                break
        return order_id

    def reindex( self, order ):
        """ reindex an order """
        return self.storage.reindex( order )

    def __contains__( self, order_id ):
        return order_id in self.storage
        
    def isValid( self, order_id ):
        try:
            int( order_id )
            return True
        except TypeError, ValueError:
            return False
            
    #################################
    # junk for z2.9 / f 1.4
    def manage_fixupOwnershipAfterAdd(self, *args):
        return

    def manage_setLocalRoles( self, *args ):
        return 


class OrderQuery( object ):
    """
    simple query construction.. it might be problematic for other storages without collapsing
    sort clauses where possible. best to minimize any query combinations in the released product.

    main interface to searching is the search method

    from getpaid.core.order import query
    from datetime import timedelta
    
    # find orders from the last week
    results = query.search( creation_date = timedelta(7) )

    """

    @staticmethod
    def search( data=None, **kw ):
        """ take a dictionary of key, value pairs, and based on available queries/indexes
        construct query and return results """
        
        results = None
        if data is None:
            data = kw
        elif data and kw:
            data.update( kw )
            
        for term in [ 'finance_state',
                      'fulfillment_state',
                      'user_id',
                      'creation_date' ]:
            term_value = data.get( term )
            if term_value is None:
                continue
            
            term_results = getattr( query, term )( term_value )
            if term_results is None: # short circuit .. default and intersection
                return []
            if results is None:
                results = term_results
            else:
                results = query.merge( results, term_results )

        # actualize to order objects
        results = query.generate( results )

        if 'no_sort' in kw:
            return results
        
        # reverse sort on creation date
        return query.sort( results, 'creation_date', reverse=True )
    
    @staticmethod
    def generate( results ):
        """ used to actualize results from ifsets to 
        """
        manager = component.getUtility( interfaces.IOrderManager )
        return ResultSet( results, manager.storage )

    @staticmethod
    def sort( results, attribute, reverse=False ):
        results = list( results )
        results.sort( lambda x,y: cmp( getattr( x, attribute ), getattr( y, attribute ) ) )
        if reverse:
            results.reverse()
        return results

    @staticmethod
    def merge( *results  ):
        return reduce( intersection, [res for res in results if res is not None] )
    
    @staticmethod
    def latest( delta = None ):
        """ query by creation date, pass in either a delta to be used from the current time
            or a tuple of start date, end date to return orders from.
        """
        if not delta:  # default to one last week ?
            delta = datetime.timedelta(7)

        if isinstance( delta, tuple ):
            value = delta
        else:
            now = datetime.datetime.now()
            value = ( now-delta, now )
        manager = component.getUtility( interfaces.IOrderManager )
        return manager.storage.apply( { 'creation_date': value } )

    creation_date = latest
    
    @staticmethod
    def finance_state( value ):
        manager = component.getUtility( interfaces.IOrderManager )
        return manager.storage.apply( { 'finance_state':( value, value ) } )
    
    @staticmethod
    def fulfillment_state( value ):
        manager = component.getUtility( interfaces.IOrderManager )
        return manager.storage.apply( {'fulfillment_state':( value, value ) } )        

    @staticmethod
    def products( *products ):
        manager = component.getUtility( interfaces.IOrderManager )
        return manager.storage.apply( {'products':products } )
    
    @staticmethod
    def user_id( value ):
        manager = component.getUtility( interfaces.IOrderManager )
        return manager.storage.apply( {'user_id':( value, value ) } )                

query = OrderQuery

class ResultSet:
    """Lazily accessed set of objects."""

    def __init__(self, uids, storage):
        self.uids = uids or []
        self.storage = storage

    def __len__(self):
        return len(self.uids)

    def __iter__(self):
        for uid in self.uids:
            yield self.storage[ str( uid ) ]

class OrderStorage( BTreeContainer ):

    def __init__( self ):
        super( OrderStorage, self).__init__()
        self.indexes = PersistentDict( {
            'products' : KeywordIndex(),
            'user_id'  : FieldIndex(),
            'processor_order_id' : FieldIndex(),
            'finance_state'   : FieldIndex(),
            'fulfillment_state' : FieldIndex(),
            'creation_date' : FieldIndex() 
            } )
        
    def query( self, **args ):
        results = self.apply( args )
        return ResultSet( results, self )

    def apply(self, query):
        results = []
        for index_name, index_query in query.items():
            index = self.indexes[index_name]
            r = index.apply(index_query)
            if r is None:
                continue
            if not r:
                # empty results
                return r
            results.append((len(r), r))

        if not results:
            # no applicable indexes, so catalog was not applicable
            return None

        results.sort() # order from smallest to largest

        _, result = results.pop(0)
        for _, r in results:
            _, result = weightedIntersection(result, r)

        return result
    
    def __setitem__( self, key, object):
        super( OrderStorage, self ).__setitem__( key, object )
        self.index( object )

    def reset_index( self ):
        # reindex all orders
        for index in self.indexes.values():
            index.clear()
        for order in self.values():
            self.index( order )

    def reindex( self, object ):
        self.unindex( object.order_id )
        self.index( object )
            
    def index( self, object ):
        doc_id = int( object.order_id )
        for attr, index in self.indexes.items():
            value = getattr( object, attr, None)
            if callable( value ):
                value = value()
            if value is None:
                continue
            index.index_doc( doc_id, value )

    def unindex( self, order_id ):
        for index in self.indexes.values():
            index.unindex_doc( int( order_id ) )
        
    def __delitem__( self, key ):
        super( OrderStorage, self).__delitem__( key )
        doc_id = int( key )
        self.unindex( doc_id )

class OrderWorkflowRecord( Persistent ):

    I = interfaces.IOrderWorkflowEntry
    FP = FieldProperty

    implements( interfaces.IOrderWorkflowEntry )

    changed_by = FP( I['changed_by'] )
    change_date = FP( I['change_date'] )
    comment = FP( I['comment'] )
    new_state = FP( I['new_state'] )
    previous_state = FP( I['previous_state'] )
    change_kind = FP(I['change_kind'])
    
    def __init__( self, **kw ):
        names = interfaces.IOrderWorkflowEntry.names()
        for k,v in kw.items():
            if k in names:
                self.__dict__[ k ] = v

class OrderWorkflowLog( object ):

    implements( interfaces.IOrderWorkflowLog )

    _store = None
    _key = "getpaid.order.auditlog"

    def __init__( self, context ):
        self.context = context

    def add( self, record ):
        self._storage().append( record )

    def _storage( self ):
        if self._store:
            return self._store

        annotation = IAnnotations( self.context )
        if annotation.has_key( self._key ):
            self._store = annotation[ self._key ]
        else:
            annotation[ self._key ] = self._store = PersistentList()
        return self._store

    def __iter__( self ):
        for record in self._storage():
            yield record


def recordOrderWorkflow( order, event ):

    data = {}

    if getSecurityManager is not None:
        data['changed_by'] = getSecurityManager().getUser().getId()

    data['change_date'] = datetime.datetime.now()
    data['new_state'] = event.destination
    data['previous_state'] = event.source
    data['transition'] = event.transition.title
    data['comment'] = event.comment

    # figure out which workflow it is to denote change kind
    if order.finance_state == event.destination:
        data['change_kind'] = _(u'Finance')
    else:
        data['change_kind'] = _(u'Fulfillment')
        
    audit_log = interfaces.IOrderWorkflowLog( event.object )
    audit_log.add( OrderWorkflowRecord( **data ) )

def reindexOrder( order, event ):
    manager = component.getUtility( interfaces.IOrderManager )
    if order.order_id in manager:
        manager.reindex( order )

def startOrderFulfillmentWorkflow( order, event ):
    """ start order and item fulfillment workflow """
    # start fufillment workflow, sends to state NEW
    order.fulfillment_workflow.fireTransition('create')
    for item in order.shopping_cart.values():
        item.fulfillment_workflow.fireTransition('create')
        

 



# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""
$Id: interfaces.py 1641 2008-06-18 11:23:25Z matt.halstead $
"""

from zope.interface import Interface, Attribute, classImplements, implements
from zope import schema
try:
    from zope.component.interfaces import IObjectEvent
except ImportError:
    # BBB for Zope 2.9
    from zope.app.event.interfaces import IObjectEvent

try:
    from zope.component.interfaces import ObjectEvent
except ImportError: 
    # BBB for Zope 2.9
    from zope.app.event.objectevent import ObjectEvent    
    
from zope.app.container.interfaces import IContainer
from zope.schema.interfaces import ITextLine
from zope.schema.vocabulary import SimpleVocabulary
from fields import PhoneNumber, CreditCardNumber, weightValidator
from zope.i18nmessageid import MessageFactory
_ = MessageFactory('getpaid')


#################################
# Where to Buy Stuff

class IStore( Interface ):
    """ represents a getpaid installation, should be a local site w/ getpaid local components installed
    """

class IPersistentOptions( Interface ):
    """
    a base interface that our persistent option annotation settings,
    can adapt to. specific schemas that want to have context stored
    annotation values should subclass from this interface, so they
    use adapation to get access to persistent settings. for example,
    settings = IMySettings(context)
    """

class IStoreSettings( IPersistentOptions ):
    """ minimum configuration schema for a store, pgp product has examples of many more.
    
    TODO: there some duplication here between PGP interfaces for store configuration and the ones here.
    the ones here are for usage without plone... ie z3 unit tests, and z3 stores. ideally the ones
    in pgp would be derived from these.. which needs parallel work to fix up translations.
    """
    
    shipping_method = schema.Choice( title = _(u"Shipping Method"),
                                    description = _(u"Select a method to calculate shipping charges for orders in your store."),
                                     required = True,
                                     source = "getpaid.shipping_methods" )

    store_name = schema.TextLine( title = _(u"Store/Organization Name"),
                                  required = True,
                                  default = u"" )

    contact_name = schema.TextLine( title = _(u"Contact Name"),
                                    required = False,
                                    default = u"" )
                                    

    contact_email = schema.TextLine( title = _(u"Contact Email"),
                                  required = False,
                                  default = u""
                                )
                                
    contact_company = schema.TextLine( title = _(u"Contact Company"),
                              required = False,
                              default = u""
                            )

    contact_address = schema.TextLine( title = _(u"Contact Address"),
                                       required = False,
                                       default = u""
                                     )
                                
    contact_address2 = schema.TextLine( title = _(u"Contact Address2"),
                                        required = False,
                                        default = u""
                                      )

    contact_city = schema.TextLine( title = _(u"Contact City"),
                                    required = False,
                                    default = u""
                                  )

    contact_country = schema.Choice( title = _(u"Contact Country"),
                                     required = False,
                                     vocabulary = "getpaid.countries"
                                   )

    contact_state = schema.Choice( title = _(u"Contact State/Province"),
                                   required = False,
                                   vocabulary = "getpaid.states"
                                 )

    contact_postalcode = schema.TextLine( title = _(u"Contact Zip/Postal Code"),
                                          required = False,
                                          default = u""
                                        )

    contact_phone = PhoneNumber( title = _(u"Contact Phone"),
                                 description = _(u"Only digits allowed"),
                                     required = False,
                                     default = u""
                                   )

    contact_fax = schema.TextLine( title = _(u"Contact Fax"),
                                   required = False,
                                   default = u""
                                 )

    tax_ein = schema.TextLine( title= _(u"Tax Identification Number"),
                               required = False,
                               default=u"")


#################################
# Plugin Management

class IPluginManager( Interface ):
    """
    a lifecycle manager for a single plugin
    """
    
    title = schema.TextLine(title=_(u"Title"))
    description = schema.TextLine(title=_(u"Description"))  
    
    def install( ):
        """install the plugin"""
        
    def uninstall( remove_data=False ):
        """ 
        uninstall the plugin, if remove data is true, the plugin
        should remove its persistent state.
        """
        
    def status():
        """ return true if installed, else false """

class IStoreInstalledEvent( IObjectEvent ):
    """ object event for store installation, and plugin installation """
    
class StoreInstalled( ObjectEvent ):
    implements( IStoreInstalledEvent )
    
class IStoreUninstallEvent( IObjectEvent ):
    """ object event for store uninstallation, and plugin removal """
    
class StoreUninstalled( ObjectEvent ):
    implements( IStoreUninstallEvent )

class IFormSchemas(Interface):
    """
    Utility for getting hold of the form schemas for a particular
    section. Any schema interface with fields that an implementor
    may want to chang should be given a section here rather than
    drectly using a hard-coded interface all over the place.
    """

    def getInterface(section):
        """
        Return the schema interface for the section specified.
        """

    def getPersistentBagClass(section):
        """
        Return the subclass of options.PersistentBag to use
        for storing field information for the specified section.
        """

#################################
# Stuff To Buy

class IPayable( Interface ):
    """
    An object which can be paid for. Payables are typically gotten via adapation between
    a context and the request, to allow for pricing / display customization on a user
    basis.
    """
    
    made_payable_by = schema.TextLine(
        title = _(u"Made Payable By"),
        readonly = True,
        required = False
        )
    
    product_code = schema.TextLine( title = _(u"Product Code"),
                        description=_(u"An organization's unique product identifier (not required since shopping cart uses content UID internally)"),
                        required=False
                        )
    price = schema.Float( title = _(u"Price"), required=True)

class IDonationContent( IPayable ):
    """ Donation
    """
    donation_text = schema.TextLine( title = _(u"Donation Description"),
                        description=_(u"Very brief 50 character text (that shows up in portlet)"),
                        required=True,
                        max_length=50)

class ISubscription( IPayable ):
    """ Subscription
    """

class IBuyableContent( IPayable ):
    """ Purchasable Content Delivered Virtually
    """
    
class IPremiumContent( IPayable ):
    """ Premium Content for Subscriptions
    """

class IPhysicalPayable( IPayable ):
    """
    """


UNIT_POUNDS = _(u"lbs")
UNIT_KILOGRAMS = _(u"kgs")

class IShippableContent( IPayable ):
    """ Shippable Content
    """
    dimensions = schema.TextLine( title = _(u"Dimensions"))
    sku = schema.TextLine( title = _(u"Product SKU"))
    
    # default unit is country of origin specific... 
    weight = schema.Float( title = _(u"Shipping Weight"),
                           constraint=weightValidator,
                           description = _(u"Enter a weight, only two decimal places are supported. The unit is specified in the field below."))
                           
    weight_unit = schema.Choice( title=_(u"Weight Unit"), values=[ UNIT_POUNDS, UNIT_KILOGRAMS ],description = _(u"Please select the unit in which you specified the shipment weight.") )
    
    def getShipWeight( self ):
        """ Shipping Weight
        """

#################################
# Events

class IPayableCreationEvent( IObjectEvent ):
    """ sent out when a payable is created
    """

    payable = Attribute("object implementing payable interface")    
    payable_interface = Attribute("payable interface the object implements")

class IPayableAuditLog( Interface ):
    """ ordered container of changes, most recent first, hook on events.
    """


#################################
# Stuff to Process Payments

class IPaymentProcessor( Interface ):
    """ A Payment Processor

    a processor can keep processor specific information on an orders
    annotations.
    """ 

    def authorize( order, payment_information ):
        """
        authorize an order, using payment information.
        """

    def capture( order, amount ):
        """
        capture amount from order.
        """

    def refund( order, amount ):
        """
        reset
        """

class IRecurringPaymentProcessor( IPaymentProcessor ):
    """ a payment processor that can handle recurring line items
    """
    
class IPaymentProcessorOptions( Interface ):
    """ Options for a Processor

    """

class IWorkflowPaymentProcessorIntegration( Interface ):
    """
    integrates an order's workflow with a payment processor
    """

    def __call__( order_workflow_event ):
        """
        process a workflow event
        """
    
#################################
# Info needed for payment processing

    
class ILineItem( Interface ):
    """
    An Item in a Cart
    """
    item_id = schema.TextLine( title = _(u"Unique Item Id"))
    name = schema.TextLine(title = _(u"Name"))
    description = schema.TextLine( title = _(u"Description"))
    cost = schema.Float( title = _(u"Cost"))
    quantity = schema.Int( title = _(u"Quantity"))
    product_code = schema.TextLine( title = _(u"Product Code"))


class ILineItemFactory( Interface ):
    """ encapsulation of creating and adding a line item to a line item container
    from a payable. sort of like an adding view
    """
    
    def create( payable ):
        """
        create a payable from a line item
        """

class ILineItemContainer( IContainer ):
    """ A container for line items
    """

class ILineContainerTotals( Interface ):
    # interface for getting prices for a collection of items (aka an order),
    # mostly encapsulation, of other components
    
    def getTotalPrice( ):
        """
        return the total price of all line items in the container
        """

    def getShippingCost( ):
        """
        return total estimated shipping cost for the items in the container.
        """

    def getTaxCost( ):
        """
        return list of dictionaries for each tax cost for the items in the container
        """

    def getSubTotalPrice( ):
        """
        get the price of all the items in the contaners
        """
    
class IPayableLineItem( ILineItem ):
    """
    A line item linked to a payable
    """

    uid = schema.ASCIILine( title = _(u"Integer Id for a Product") )

    def resolve( ):
        """ return the payable object, or None if can't be found.
        """

class IShippableLineItem( ILineItem ):
    """
    a line item that can be shipped
    """
    #implements( interfaces.IShippableLineItem )
    weight = schema.Float( title = _(u'Weight of Packaged Item'),
                           required = True,
                           )
    #um_weight = 
    #um_distance = ""
    length = schema.Float( title=_(u"Length"))
    height = schema.Float( title=_(u"Height"))
    width = schema.Float( title=_(u"Width"))    
        
class IRecurringLineItem( IPayableLineItem ):

    period = schema.Int( title = _(u"Period as a timedelta"))
    

class IGiftCertificate( ILineItem ):
    """ A Gift Certificate
    """

#################################
# Shopping Cart Stuff

class IShoppingCartUtility( Interface ):

    def get( context, create=False, key=None ):
        """
        Return the user's shopping cart or none if not found. If
        create is passed then create a new one if one isn't found. If
        key is passed then return the cart corresponding to that key
        independent of the current user.
        """

    def destroy( context, key=None ):
        """
        Remove the current user's cart from the session if it exists.
        If key is passed then remove the cart corresponding to that
        key independent of the current user.
        """

    def getKey( context ):
        """
        Return a key for the shopping cart of the current user
        including anonymous users. This key can then be used to
        retrieve or destroy the cart at a later point. This is to
        support handling of notification callbacks with async
        processors.
        """

class IShoppingCart( ILineItemContainer ):
    """ A Shopping Cart 
    """
    def size( ):
        """
        Count the number of items in the cart (*not* the number of line
        items)
        """
        
#################################
# Shipping

class IShipmentContainer(  IContainer ):
    """ a container for shipments
    """

class IShipment( ILineItemContainer ):
    """ a (partial|complete) shipment of ishippable line items of an order
    """
    tracking_code = schema.TextLine( description=_(u"Tracking Number") )
    service_code = schema.ASCIILine( description=_(u"Service Code (2 Letter)"))
    service = schema.TextLine( description=_(u"Service Name"))
    shipping_cost = schema.Float( description=_(u"Cost of Delivery") )
    creation_date = schema.Datetime( description=_(u"Creation Date") )

class IShippingMethod( Interface ):

    def getCost( order ):
        """ get the shipping cost for an order...
        """

class IShippingRateService( Interface ):
    """ utility """

    def getRates( order ):
        """ return shipping rate options for an order.  this should return:
        - a list of IShippingMethodRate as 'shipments'
        - an error string as 'error'
        """

    def getMethodName( method_id ):
        """
        given a shipping method service code, returns a shipping method label/name
        """

    def getTrackingUrl( track_number ):
        """
        given a track number this should return, if available for this service
        a url that can be used to track the shipment
        """

class IShippingMethodRate( Interface ):
    """
     Service Code: UPS Next Day Air
     Shipment unit of measurement: LBS
     Shipment weight: 3.0
     Currency Code: USD
     Total Charge: 58.97
     Days to Delivery: 1
     Delivery Time: 10:30 A.M.
    """
    # may need to find a way to make this general
    service_code = schema.ASCIILine( description=_(u"Service Code (2 Letter)"))
    service = schema.TextLine( description=_(u"Service Name"))

    currency = schema.ASCII( description=_(u"Currency Denomination Code"))
    cost = schema.Float( description=_(u"Cost of Delivery"))

    # really shouldn't show these, as they ignore store processing time
    # days_to_delivery = schema.Int( description=_(u"Estimated Days to Deliver") )
    # delivery_time = schema.TextLine( description=_(u"Estimated Delivery Time") )

class IShippingMethodSettings( Interface ):
    """ Options for a Shipping Method
    """

#################################
# Tax Utility

class ITaxUtility( Interface ):

    def getTaxes( order ):
	 	""" return a list dictionaries of each ITax inside that applies to
	 		the order
		"""
        

#################################
# Payment Information Details
class IAbstractAddress( Interface ):
    """ base/common interface for all addresses"""
    
class IAddress( IAbstractAddress ):
    """ a physical address
    """
    first_line = schema.TextLine( title = _(u"Address 1"), description=_(u"Please Enter Your Address"))
    second_line = schema.TextLine( title = _(u"Address 2"), required=False )
    city = schema.TextLine( title = _(u"City") )
    country = schema.Choice( title = _(u"Country"),
                               vocabulary = "getpaid.countries")    
    state = schema.Choice( title = _(u"State"),
                             vocabulary="getpaid.states")
    postal_code = schema.TextLine( title = _(u"Zip/Postal Code"))

class IShippingAddress( IAbstractAddress ):
    """ where to send goods
    """
    ship_same_billing = schema.Bool( title = _(u"Same as billing address"), required=False)
    ship_name = schema.TextLine( title = _(u"Full Name"), required=False)
    ship_first_line = schema.TextLine( title = _(u"Address 1"), required=False)
    ship_second_line = schema.TextLine( title = _(u"Address 2"), required=False)
    ship_city = schema.TextLine( title = _(u"City"), required=False)
    ship_country = schema.Choice( title = _(u"Country"),
                                    vocabulary = "getpaid.countries", required=False)
    ship_state = schema.Choice( title = _(u"State"),
                                  vocabulary="getpaid.states", required=False)
    ship_postal_code = schema.TextLine( title = _(u"Zip/Postal Code"), required=False)

class IBillingAddress( IAbstractAddress ):
    """ where to bill 
    """
    bill_name = schema.TextLine( title = _(u"Full Name"))
    bill_first_line = schema.TextLine( title = _(u"Address 1"))
    bill_second_line = schema.TextLine( title = _(u"Address 2"), required=False )
    bill_city = schema.TextLine( title = _(u"City") )
    bill_country = schema.Choice( title = _(u"Country"),
                                    vocabulary = "getpaid.countries")
    bill_state = schema.Choice( title = _(u"State"),
                                  vocabulary="getpaid.states" )
    bill_postal_code = schema.TextLine( title = _(u"Zip/Postal Code"))

MarketingPreferenceVocabulary = SimpleVocabulary( 
                                   map(SimpleVocabulary.createTerm, 
                                       ( (True, "Yes", _(u"Yes")), (False, "No", _(u"No") ) )
                                       )
                                )
                                
EmailFormatPreferenceVocabulary = SimpleVocabulary( 
                                   map( lambda x: SimpleVocabulary.createTerm(*x), 
                                       ( (True, "Yes", _(u"HTML")), (False, "No", _(u"Plain Text") ) )
                                       )
                                  )                                

class IUserContactInformation( Interface ):
    """docstring for IUserContactInformation"""
    
    name = schema.TextLine( title = _(u"Your Name"))
    
    phone_number = PhoneNumber( title = _(u"Phone Number"),
                                description = _(u"Only digits allowed - e.g. 3334445555 and not 333-444-5555 "))
                                
    email = schema.TextLine( 
                        title=_(u"Email"),
                        description = _(u"Contact Information") 
                        )
        
    marketing_preference = schema.Bool(
                                        title=_(u"Can we contact you with offers?"), 
                                        required=False,
                                        ) 
    
    email_html_format = schema.Choice( 
                                        title=_(u"Email Format"), 
                                        description=_(u"Would you prefer to receive rich html emails or only plain text"),
                                        vocabulary = EmailFormatPreferenceVocabulary
                                        )

                                
class IUserPaymentInformation( Interface ):
    """ A User's payment information to be optionally collected by the
    payment processor view.
    """

    name_on_card = schema.TextLine( title = _(u"Card Holder Name"),
                                description = _(u"Enter the full name, as it appears on the card. "))

    bill_phone_number = PhoneNumber( title = _(u"Phone Number"),
                                description = _(u"Only digits allowed - e.g. 3334445555 and not 333-444-5555 "))

    # DONT STORED PERSISTENTLY
    credit_card_type = schema.Choice( title = _(u"Credit Card Type"),
                                      source = "getpaid.core.accepted_credit_card_types",)

    credit_card = CreditCardNumber( title = _(u"Credit Card Number"),
                                    description = _(u"Only digits allowed - e.g. 4444555566667777 and not 4444-5555-6666-7777 "))

    cc_expiration = schema.Date( title = _(u"Credit Card Expiration Date"),
                                    description = _(u"Select month and year"))

    cc_cvc = schema.TextLine(title = _(u"Credit Card Verfication Number"),
                             description = _(u"For MC, Visa, and DC, this is a 3-digit number on back of the card.  For AmEx, this is a 4-digit code on front of card. "),
                             min_length = 3,
                             max_length = 4)

#################################
#
class IProductCatalog( Interface ):
    
    def query( **kw ):
        """ query products """
    def __setitem__( product_id, product ):
        """ """
#################################
# Orders

class IOrderManager( Interface ):
    """ persistent utility for storage and query of orders
    """

    def query( **kw ):
        """ query the orders, XXX extract order query interface
        """

    def get( order_id ):
        """ retrieve an order
        """

    def reindex( order ):
        """ reindex a modified order
        """
# future interface
#    def __setitem__( order_id, order ):
#               """ save an order
#        """

    def store( order ):
         """ save an order
         """
class IDefaultFinanceWorkflow( Interface ):
    """ marker interface for workflow / processor integration on the default ootb workflow """

class IOrder( Interface ):
    """ captures information, and is a container to multiple workflows
    """
    user_id = schema.ASCIILine( title = _(u"Customer Id"), readonly=True )
    shipping_address = schema.Object( IShippingAddress, required=False)
    billing_address  = schema.Object( IBillingAddress )
    # only shown on anonymous checkouts?
    contact_information = schema.Object( IUserContactInformation, required=False )
    shopping_cart = schema.Object( IShoppingCart )
    finance_state = schema.TextLine( title = _(u"Finance State"), readonly=True)
    fulfillment_state = schema.TextLine( title = _(u"Fulfillment State"), readonly=True)
    processor_order_id = schema.ASCIILine( title = _(u"Processor Order Id") )
    processor_id = schema.ASCIILine( readonly=True )

# Various Order Classification Markers..

class IShippableOrder( IOrder ):
    """ marker interface for orders which need shipping """
    
    shipping_service = schema.ASCIILine( title = _(u"Shipping Service"))
    shipping_method = schema.ASCIILine( title = _(u"Shipping Method") )
    shipping_price = schema.ASCIILine( title = _(u"Shipping Price") )

    shipments = schema.Object( IShipmentContainer )
    

class IOriginRouter( Interface ):
    
    def getOrigin( ):
        """
        determine the origin shipping point for an order..
        
        return an IContactInfo and IAddress object to serve as the origin
        of the order for any fufillment process (shipping, processing).
        """    
        
class IRecurringOrder( IOrder ):
    """ marker interface for orders containing recurring line items """

class IVirtualOrder( IOrder ):
    """ marker inteface for orders which are delivered virtually """

class IDonationOrder( IOrder ):
    """ marker interface for orders which contain donations"""

class IOrderSetReport( Interface ):
    """ store adapters that can serialize a set of orders into a report"""
    
    title = schema.TextLine()
    mime_type = schema.ASCIILine()

    def __call__( orders ):
        """ 
        return a rendered report string from the given ordrs
        """
    
class IOrderWorkflowLog( Interface ):
    """ an event log based history of an order's workflow
    """
    def __iter__( ):
        """ iterate through records of the order's history, latest to oldest.
        """

    def last( ):
        """ get the last change to the order
        """

class IOrderWorkflowEntry( Interface ):
    """ a record describing a change in an order's workflow history
    """
    changed_by = schema.ASCIILine( title = _(u"Changed By"), readonly = True )
    change_date = schema.Date( title = _(u"Change Date"), readonly = True)
    change_kind = schema.TextLine( title=_(u"Change Kind"), readonly=True) 
    comment = schema.ASCIILine( title = _(u"Comment"), readonly = True, required=False )
    new_state = schema.ASCIILine( title = _(u"New State"), readonly = True)
    previous_state = schema.ASCIILine( title = _(u"Previous State"), readonly = True )
    transition = schema.ASCIILine( title = u"", readonly = True)
    # change type?? (workflow, user


class IPhoneNumber(ITextLine):
    """A Text line field that handles phone number input."""
    
classImplements(PhoneNumber,IPhoneNumber)


class ICreditCardNumber(ITextLine):
    """A Text line field that handles credit card input."""
classImplements(CreditCardNumber,ICreditCardNumber)

class ICreditCardTypeEnumerator(Interface):
    """Responsible for listing credit card types. """

    def acceptedCreditCardTypes(self):
        """ Lists the accepted credit card types. """

    def allCreditCardTypes(self):
        """ List all credit card types. """

class keys:
    """ public annotation keys and static variables
    """

    # how much of the order have we charged
    capture_amount= 'getpaid.capture_amount'
    
    # processor specific txn id for an order
    processor_txn_id = 'getpaid.processor.uid'
    
    # name of processor adapter
    processor_name = 'getpaid.processor.name'

    # sucessful call to a processor
    results_success = 1
    results_async = 2
    
class workflow_states:

    class order:
        # order workflows are executed in parallel

        class finance:
            # name of parallel workflow            
            name = "order.finance"

            REVIEWING = 'REVIEWING'
            CHARGEABLE = 'CHARGEABLE'
            CHARGING = 'CHARGING'
            CHARGED = 'CHARGED'
            REFUNDED = 'REFUNDED'
            PAYMENT_DECLINED = 'PAYMENT_DECLINED'
            CANCELLED = 'CANCELLED'
            CANCELLED_BY_PROCESSOR = 'CANCELLED_BY_PROCESSOR'
            
        class fulfillment:
            # name of parallel workflow
            name = "order.fulfillment"            

            NEW = 'NEW'
            PROCESSING = 'PROCESSING'
            DELIVERED = 'DELIVERED'
            WILL_NOT_DELIVER = 'WILL_NOT_DELIVER'

    class shippable_order:
          
        class fulfillment:
            # name of parallel workflow
            name = "order.fulfillment"            

            NEW = 'NEW'
            PROCESSING = 'PROCESSING'
            #This is actually The same as delivered for order,
            DELIVERED = 'SHIPPED'
            WILL_NOT_DELIVER = 'WILL_NOT_DELIVER'


            
    class item:
        NEW = 'NEW'
        PROCESSING = 'PROCESSING'
        DELIVER_VIRTUAL = 'DELIVERVIRTUAL'
        CANCELLED = 'CANCELLED'
        SHIPPED = 'SHIPPED'
        #RETURNING = 'RETURNING'
        #RETURNED = 'RETURNED'
        REFUNDING = 'REFUNDING'
        REFUNDED = 'REFUNDED'

    class shipment:
        NEW = 'NEW'
        CHARGING = 'CHARGING'
        DECLINED = 'DECLINED'
        DELIVERED = 'DELIVERED'
        SHIPPED = 'SHIPPED'
        SHIPPABLE = 'SHIPPABLE'
        CHARGED = 'CHARGED'
            



# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

from zope import schema
from zope.schema.interfaces import ValidationError

from zope.i18nmessageid import MessageFactory
_ = MessageFactory('getpaid')

class InvalidPhoneNumber(ValidationError):
    __doc__ = _(u"Only digit character allowed.")

class InvalidCreditCardNumber(ValidationError):
    __doc__ = _(u"Invalid Credit Card Number.")

class InvalidWeight( ValidationError ):
    __doc__ = _(u"Invalid Weight")
    
def weightValidator( weight ):
    if weight <= 0:
        raise InvalidWeight( weight )
    return True
    
def creditCardValid(card_number):
    """ checks to make sure that the card passes a luhn mod-10 checksum """
    # strip any whitespace
    card_number = card_number.replace(' ', '').strip()
    
    if isinstance( card_number, unicode ) and not card_number.isnumeric():
        return False
    
    elif isinstance( card_number, str) and not card_number.isdigit():
        return False

    sum = 0
    num_digits = len(card_number)
    oddeven = num_digits & 1
    for count in range( 0, num_digits):
        digit = int(card_number[count])
        if not (( count & 1 ) ^ oddeven ):
            digit = digit * 2
        if digit > 9:
            digit = digit - 9
        sum = sum + digit
    return ( (sum % 10) == 0 )

class PhoneNumber( schema.TextLine):

    def _validate(self, value):
        super(PhoneNumber, self)._validate(value)
        if value and not value.isdigit():
            raise InvalidPhoneNumber(value)

class CreditCardNumber( schema.TextLine ):

    def _validate(self, value):
        super(CreditCardNumber, self)._validate(value)
        if not creditCardValid( value ):
            raise InvalidCreditCardNumber(value)



# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""Doc test runner
"""

__docformat__ = "reStructuredText"

import doctest
import unittest
from zope.testing.doctestunit import DocFileSuite
from zope.app.testing import placelesssetup, ztapi

try:
    from zope.annotation import interfaces as annotation_interfaces
    from zope.annotation import attribute
except ImportError:
    # BBB for Zope 2.9
    from zope.app.annotation import interfaces as annotation_interfaces
    from zope.app.annotation import attribute

from hurry.workflow import interfaces

import random, string, datetime

from zope import component

from getpaid.core.interfaces import IOrder, IOrderManager, IOrderWorkflowLog, workflow_states
from getpaid.core import order, cart, item as line_item
from getpaid.core.workflow import store, order as oworkflow
from getpaid.core import payment

class GetPaidTestCase( unittest.TestCase ):
    
    def setUp( self ):
        coreSetUp( )
        super(GetPaidTestCase, self).setUp()

    def tearDown( self ):
        placelesssetup.tearDown()

def createOrders( how_many=10 ):
    manager = component.getUtility( IOrderManager )

    for i in range(1, how_many):
        o = order.Order()
        o.order_id = str(i)

        o.shopping_cart = sc = cart.ShoppingCart()
        
        for i in range(0, 10):
            item = line_item.LineItem()
            item.name = "p%s"%random.choice( string.letters )
            item.quantity = random.randint(1,25)
            item.cost = random.randint(30, 100)
            item.item_id = "i%s"%random.choice( string.letters )
            if item.item_id in sc:
                continue
            sc[item.item_id] = item
            
        o.user_id = "u%s"%random.choice( string.letters )
        #o.finance_workflow.fireTransition('create')
        #o.fulfillment_workflow.fireTransition('create')
        
        manager.store( o )

        yield o

def coreSetUp( test=None ):
    placelesssetup.setUp()

    ###########################
    # order workflow
    ztapi.provideAdapter( IOrder,
                          interfaces.IWorkflowState,
                          oworkflow.FinanceState,
                          'order.finance')

    ztapi.provideAdapter( IOrder,
                          interfaces.IWorkflowState,
                          oworkflow.FulfillmentState,
                          'order.fulfillment')

    ztapi.provideAdapter( IOrder,
                          interfaces.IWorkflowInfo,
                          oworkflow.FinanceInfo,
                          'order.finance')

    ztapi.provideAdapter( IOrder,
                     interfaces.IWorkflowInfo,
                     oworkflow.FulfillmentInfo,
                    'order.fulfillment')
    
    ztapi.provideAdapter(annotation_interfaces.IAttributeAnnotatable,
                         annotation_interfaces.IAnnotations,
                         attribute.AttributeAnnotations)

    ztapi.provideUtility(interfaces.IWorkflow,
                         oworkflow.FulfillmentWorkflow(),
                        'order.fulfillment')
                        
    ztapi.provideUtility(interfaces.IWorkflow,
                        oworkflow.FinanceWorkflow(),
                        'order.finance')

    ztapi.provideUtility(interfaces.IWorkflowVersions,
                         store.StoreVersions())
    
    ztapi.provideUtility( IOrderManager, order.OrderManager() )

    ztapi.provideAdapter( IOrder, IOrderWorkflowLog, order.OrderWorkflowLog )

    ztapi.subscribe( (IOrder, interfaces.IWorkflowTransitionEvent), 
                       None, 
                       order.recordOrderWorkflow )

    ######################
    # product catalog




# Copyright (c) 2007 ifPeople, Kapil Thangavelu, and Contributors
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation
# files (the "Software"), to deal in the Software without
# restriction, including without limitation the rights to use,
# copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following
# conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
# OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
# OTHER DEALINGS IN THE SOFTWARE.

"""
product catalog
"""


from zope.index.field import FieldIndex
from zope.index.keyword  import KeywordIndex
from zope.app.intids.interfaces import IIntIds
from zope import component, interface
from persistent import Persistent

from getpaid.core import interfaces, catalog, options

def productModified( product, event ):
    products = component.getUtility( interfaces.IProductCatalog )
    products.reindex( product.uid, product )
    
def productDeleted( product, event ):
    products = component.getUtility( interfaces.IProductCatalog )
    products.unindex( product.uid, product )
    products[ product.uid ] = 'deleted'
    
class ProductBag( Persistent, options.PropertyBag): pass

ProductBag.initclass( interfaces.IPayable )

class ProductQuery( catalog.RecordQuery ):

    @classmethod
    def getStorage( cls ):
        catalog = component.getUtility( interfaces.IProductCatalog )
        return catalog

class ProductCatalog( catalog.IndexedRecords ):
    
    interface.implements( interfaces.IProductCatalog )
    
    index_map = dict(
         product_id = (FieldIndex, 'field'),
         categories = (KeywordIndex, 'key'),
         featured = (FieldIndex, 'field'),
         price = (FieldIndex, 'field'),
         deleted = (FieldIndex, 'field')
         )
         
    def __getitem__( self, key ):
        intids = component.getUtility( IIntIds )
        return intids.queryObject( int(key) ) or self[ key ]
        
    def values( self ):
        intids = component.getUtility( IIntIds )
        for k in self.keys():
            ob = intids.queryObject( int( k ) )
            if ob:
                yield ob
            else:
                yield self[ k ]
            
    def __setitem__( self, key, payable ):
        bag = ProductBag.frominstance( payable )
        super( ProductCatalog, self).__setitem__( key, bag )
